@@ -21,8 +21,8 @@
 #ifndef DEFAULTTAGSTEST_H
 #define DEFAULTTAGSTEST_H
 
-#include <QtTest>
-#include <QtCore/QObject>
+#include <QtTest/QTest>
+#include <QtCore/QDebug>
 
 #include "template.h"
 #include "engine.h"
@@ -165,6 +165,16 @@ private Q_SLOTS:
     doTest();
   }
 
+  void testDebugTag_data();
+  void testDebugTag() {
+    doTest();
+  }
+
+  void testLoadTag_data();
+  void testLoadTag() {
+    doTest();
+  }
+
   void testUrlTypes_data();
   void testUrlTypes();
 
@@ -222,6 +232,10 @@ void TestDefaultTags::doTest()
   QCOMPARE( t->error(), NoError );
 
   QCOMPARE( result, output );
+
+  if ( !result.isEmpty() && QString::fromLatin1( QTest::currentTestFunction() ) == QLatin1String( "testMediaFinderTag" ) ) {
+    QVERIFY( !context.externalMedia().isEmpty() );
+  }
 }
 
 void TestDefaultTags::testCommentTag_data()
@@ -240,6 +254,7 @@ void TestDefaultTags::testCommentTag_data()
   QTest::newRow( "comment-tag03" ) << QString::fromLatin1( "foo{% comment %} {% if %} {% endcomment %}" ) << dict << QString::fromLatin1( "foo" ) << NoError;
   QTest::newRow( "comment-tag04" ) << QString::fromLatin1( "foo{% comment %} {% endblock %} {% endcomment %}" ) << dict << QString::fromLatin1( "foo" ) << NoError;
   QTest::newRow( "comment-tag05" ) << QString::fromLatin1( "foo{% comment %} {% somerandomtag %} {% endcomment %}" ) << dict << QString::fromLatin1( "foo" ) << NoError;
+  QTest::newRow( "comment-tag06" ) << QString::fromLatin1( "{% comment %}yes" ) << dict << QString() << UnclosedBlockTagError;
 }
 
 void TestDefaultTags::testFirstOfTag_data()
@@ -556,6 +571,8 @@ void TestDefaultTags::testIfTag_data()
   QTest::newRow( "if-tag-error03" ) << QString::fromLatin1( "{% if foo or %}yes{% else %}no{% endif %}" ) << dict << QString() << TagSyntaxError;
   QTest::newRow( "if-tag-error04" ) << QString::fromLatin1( "{% if not foo and %}yes{% else %}no{% endif %}" ) << dict << QString() << TagSyntaxError;
   QTest::newRow( "if-tag-error05" ) << QString::fromLatin1( "{% if not foo or %}yes{% else %}no{% endif %}" ) << dict << QString() << TagSyntaxError;
+  QTest::newRow( "if-tag-error06" ) << QString::fromLatin1( "{% if %}yes{% else %}no{% endif %}" ) << dict << QString() << TagSyntaxError;
+  QTest::newRow( "if-tag-error07" ) << QString::fromLatin1( "{% if foo %}yes" ) << dict << QString() << UnclosedBlockTagError;
 
   // Truthiness
   dict.clear();
@@ -651,6 +668,10 @@ void TestDefaultTags::testForTag_data()
   list.clear();
   dict.clear();
 
+  QTest::newRow( "for-tag07" ) << QString::fromLatin1( "{% for %}{% endfor %}" ) << dict << QString() << TagSyntaxError;
+  QTest::newRow( "for-tag08" ) << QString::fromLatin1( "{% for foo bar bat %}{% endfor %}" ) << dict << QString() << TagSyntaxError;
+  QTest::newRow( "for-tag09" ) << QString::fromLatin1( "{% for foo bar '' %}{% endfor %}" ) << dict << QString() << TagSyntaxError;
+
   list << 1 << 2 << 3;
   dict.insert( QLatin1String( "values" ), list );
   QTest::newRow( "for-tag-vars01" ) << QString::fromLatin1( "{% for val in values %}{{ forloop.counter }}{% endfor %}" ) << dict << QString::fromLatin1( "123" ) << NoError;
@@ -871,7 +892,7 @@ void TestDefaultTags::testIfEqualTag_data()
   dict.clear();
   dict.insert( QLatin1String( "x" ), 2 );
 
-  QTest::newRow( "ifequal-numeric07" ) << QString::fromLatin1( "{% ifequal x 2. %}yes{% endifequal %}" ) << dict << QString() << NoError;
+  QTest::newRow( "ifequal-numeric07" ) << QString::fromLatin1( "{% ifequal x 2. %}yes{% endifequal %}" ) << dict << QString() << TagSyntaxError;
 
   dict.clear();
   dict.insert( QLatin1String( "x" ), 5 );
@@ -918,6 +939,8 @@ void TestDefaultTags::testIfEqualTag_data()
 
   QTest::newRow( "ifequal-filter05" ) << "{% ifequal x|slice:\"1\"|upper \"A\" %}x{% endifequal %}" << dict << QString::fromLatin1( "x" ) << NoError;
 
+  QTest::newRow( "ifequal-error01" ) << "{% ifequal x|slice:\"1\"|upper %}x{% endifequal %}" << dict << QString() << TagSyntaxError;
+
 }
 
 void TestDefaultTags::testIfNotEqualTag_data()
@@ -1033,6 +1056,15 @@ void TestDefaultTags::testCycleTag_data()
   dict.insert( QLatin1String( "one" ), QLatin1String( "A" ) );
   dict.insert( QLatin1String( "two" ), QLatin1String( "2" ) );
   QTest::newRow( "cycle16" ) << QString::fromLatin1( "{% cycle one|lower two as foo %}{% cycle foo %}" ) << dict << QString::fromLatin1( "a2" ) << NoError;
+
+  QTest::newRow( "cycle17" ) << QString::fromLatin1( "{% cycle %}" ) << dict << QString() << TagSyntaxError;
+  QTest::newRow( "cycle18" ) << QString::fromLatin1( "{% cycle var %}" ) << dict << QString() << TagSyntaxError;
+
+  dict.insert( QLatin1String( "three" ), QLatin1String( "B" ) );
+  dict.insert( QLatin1String( "four" ), QLatin1String( "4" ) );
+
+  QTest::newRow( "cycle19" ) << QString::fromLatin1( "{% cycle one two three foo %}" ) << dict << QString::fromLatin1( "A" ) << NoError;
+  QTest::newRow( "cycle20" ) << QString::fromLatin1( "{% cycle one two as foo %}{% cycle three four as bar %}" ) << dict << QString::fromLatin1( "AB" ) << NoError;
 }
 
 void TestDefaultTags::testWidthRatioTag_data()
@@ -1325,6 +1357,7 @@ void TestDefaultTags::testIfChangedTag_data()
   dict.insert( QLatin1String( "numx" ), QVariantList() << 2 << 2 << 2 );
   dict.insert( QLatin1String( "numy" ), QVariantList() << 3 << 3 << 3 );
   QTest::newRow( "ifchanged07" ) << QString::fromLatin1( "{% for n in num %}{% ifchanged %}{{ n }}{% endifchanged %}{% for x in numx %}{% ifchanged %}{{ x }}{% endifchanged %}{% for y in numy %}{% ifchanged %}{{ y }}{% endifchanged %}{% endfor %}{% endfor %}{% endfor %}" ) << dict << QString::fromLatin1( "1233323332333" ) << NoError;
+  QTest::newRow( "ifchanged08" ) << QString::fromLatin1( "{% ifchanged %}{{ num.0 }}{% endifchanged %}" ) << dict << QString::fromLatin1( "1" ) << NoError;
 
   // datalist': [[(1, 'a'), (1, 'a'), (0, 'b'), (1, 'c')], [(0, 'a'), (1, 'c'), (1, 'd'), (1, 'd'), (0, 'e')]]}
   dict.clear();
@@ -1454,6 +1487,10 @@ void TestDefaultTags::testAutoescapeTag_data()
   dict.clear();
   dict.insert( QLatin1String( "var" ), QVariant() );
   QTest::newRow( "autoescape-tag08" ) << "{% autoescape on %}{{ var|default_if_none:\"endquote\\\" hah\" }}{% endautoescape %}" << dict << "endquote\" hah" << NoError;
+
+  QTest::newRow( "autoescape-tag09" ) << "{% autoescape on extra %}{{ var|default_if_none:\"endquote\\\" hah\" }}{% endautoescape %}" << dict << "" << TagSyntaxError;
+  QTest::newRow( "autoescape-tag10" ) << "{% autoescape bad %}{{ var|default_if_none:\"endquote\\\" hah\" }}{% endautoescape %}" << dict << "" << TagSyntaxError;
+
   // Objects which return safe strings as their __unicode__ method
   // won't get double-escaped.
 //   'autoescape-tag09': (r'{{ unsafe }}', {'unsafe': filters.UnsafeClass()}, 'you &amp; me'),
@@ -1464,6 +1501,7 @@ void TestDefaultTags::testAutoescapeTag_data()
   dict.clear();
   dict.insert( QLatin1String( "first" ), QLatin1String( "<a>" ) );
   QTest::newRow( "autoescape-filtertag01" ) << QString::fromLatin1( "{{ first }}{% filter safe %}{{ first }} x<y{% endfilter %}" ) << dict << QString() << TagSyntaxError;
+  QTest::newRow( "autoescape-filtertag02" ) << QString::fromLatin1( "{{ first }}{% filter escape %}{{ first }} x<y{% endfilter %}" ) << dict << QString() << TagSyntaxError;
 }
 
 
@@ -1520,6 +1558,36 @@ void TestDefaultTags::testRangeTag_data()
   QTest::newRow( "range-tag04" ) << QString::fromLatin1( "{% range values.0 values.1 values.2 as i %}{{ i }};{% endrange %}" ) << dict << QString::fromLatin1( "10;12;14;" ) << NoError;
 
   QTest::newRow( "range-tag05" ) << QString::fromLatin1( "{% range 5 %}Foo;{% endrange %}" ) << dict << QString::fromLatin1( "Foo;Foo;Foo;Foo;Foo;" ) << NoError;
+  QTest::newRow( "range-tag06" ) << QString::fromLatin1( "{% range 5 6 %}Foo;{% endrange %}" ) << dict << QString() << TagSyntaxError;
+  QTest::newRow( "range-tag07" ) << QString::fromLatin1( "{% range 5 6 7 %}Foo;{% endrange %}" ) << dict << QString() << TagSyntaxError;
+}
+
+void TestDefaultTags::testDebugTag_data()
+{
+
+  QTest::addColumn<QString>( "input" );
+  QTest::addColumn<Dict>( "dict" );
+  QTest::addColumn<QString>( "output" );
+  QTest::addColumn<Grantlee::Error>( "error" );
+
+  Dict dict;
+
+  QTest::newRow( "debug-tag01" ) << QString::fromLatin1( "{% debug %}" ) << dict << QString::fromLatin1( "\n\nContext:\nEnd context:\n\n" ) << NoError;
+  dict.insert( QLatin1String( "answer" ), 42 );
+  QTest::newRow( "debug-tag02" ) << QString::fromLatin1( "{% debug %}" ) << dict << QString::fromLatin1( "\n\nContext:\nkey answer, type int\nEnd context:\n\n" ) << NoError;
+}
+
+void TestDefaultTags::testLoadTag_data()
+{
+
+  QTest::addColumn<QString>( "input" );
+  QTest::addColumn<Dict>( "dict" );
+  QTest::addColumn<QString>( "output" );
+  QTest::addColumn<Grantlee::Error>( "error" );
+
+  Dict dict;
+
+  QTest::newRow( "load-tag01" ) << QString::fromLatin1( "{% load does_not_exist %}foo" ) << dict << QString() << TagSyntaxError;
 }
 
 void TestDefaultTags::testUrlTypes_data()
